@@ -1,6 +1,5 @@
 from io import BytesIO
 
-from PIL import Image, ImageFile
 from easy_thumbnails import utils
 
 
@@ -18,6 +17,8 @@ def pil_image(source, exif_orientation=True, **options):
     # object, PIL may have problems with it. For example, some image types
     # require tell and seek methods that are not present on all storage
     # File objects.
+    from PIL import Image, ImageFile
+
     if not source:
         return
     source = BytesIO(source.read())
@@ -33,3 +34,18 @@ def pil_image(source, exif_orientation=True, **options):
     if exif_orientation:
         image = utils.exif_orientation(image)
     return image
+
+
+def vil_image(source, **options):
+    """
+    Try to open the source file directly using VIL, ignoring any errors.
+    """
+    from easy_thumbnails.VIL import Image
+
+    if not source:
+        return
+    filename = source.source_storage.path(source.file.name)
+    try:
+        return Image.load(filename)
+    except Exception as exc:
+        raise exc
